# This config file serves as a template and should not be modified directly; please make any necessary changes in config.py.

# Font
font_family = ["JetBrains Mono", "Cascadia Code", "Consolas", "Menlo", "monospace"]
font_size = 11

# That's it for now.
